package org.astakhova.data;

import java.util.List;

/**
 * Makes a description of a node
 * @author Astakhova Julia A.
 */
public interface INode {
	public int getX();
	public int getY();
	public int getHeight();
	public int getWidth();
	public String getText();
	
	/**
	 * Gets a list of arrows that go out of the node
	 * @return a list of arrows that go out of the node
	 */
	public List<IArrow> getOutArrows();
	
	/**
	 * Checks if a node can be bound by an arrow
	 * @return if a node can be bound by an arrow
	 */
	public boolean canBeBound();
	
	/**
	 * Checks if a new arrow can be added to
	 *  this node's list of out arrows
	 * @return if a new arrow can be added to
	 *  this node's list of out arrows
	 */
	public boolean canBind();
	
	public void setX(int x);
	public void setY(int y);
	public void setHeight(int height);
	public void setWidth(int width);
	public void setText(String text);
}
