package org.astakhova.data.impl;

import java.util.LinkedList;
import java.util.Collections;
import java.util.List;

import org.astakhova.data.*;

/**
 * Makes a description of a diagram 
 * @author Astakhova Julia A.
 */
public class Diagram implements IDiagram{
	private LinkedList<INode> myNodeList = new LinkedList<INode>();
	
	public IDataNode createDataNode(int x, int y){
		DataNode dn = new DataNode(x,y);
		myNodeList.addLast(dn);
		return dn;
	}
	
	public IConditionNode createCondNode(int x, int y){
		ConditionNode cn = new ConditionNode(x,y);
		myNodeList.addLast(cn);
		return cn;
	}
	
	public IBeginNode createBeginNode(int x, int y){
		for (INode n : myNodeList)
			if (n instanceof IBeginNode)
				return null;
		BeginNode bn = new BeginNode(x,y);
		myNodeList.addLast(bn);
		return bn;
	}
	
	public IEndNode createEndNode(int x, int y){
		EndNode en = new EndNode(x,y);
		myNodeList.addLast(en);
		return en;
	}
	
	public IArrow createArrow(INode source, INode dest) {
		return ((Node) source).connect((Node) dest);
	}
	
	public IArrow createArrow(IConditionNode source, INode dest, 
			boolean branch) {
		return (branch ? 
				((ConditionNode) source).connectTrueBranch((Node) dest) : 
				((ConditionNode) source).connectFalseBranch((Node) dest));
	}
	
	public IArrow createArrow(IConditionNode source, INode dest, 
			boolean branch, String s) {
		IArrow a = branch ? 
				((ConditionNode) source).connectTrueBranch((Node) dest) : 
				((ConditionNode) source).connectFalseBranch((Node) dest);
		a.setLabel(s);
		return a;
	}
	
	public void removeNode(INode n) {
		((Node) n).unBind();
		myNodeList.remove(n);
	}
	
	public List<INode> getNodes() {
		return Collections.unmodifiableList(myNodeList);
	}
	
	public INode getNode(int x, int y) {
		for (INode n : myNodeList)
			if (n.getX() == x && n.getY() == y)
				return n;
		return null;
	}
	
	public String toString() {
		String s = "The diagram description:\n";
		for (INode n : myNodeList)
			s += n + "\n";
		return s;
	}
	
	public void removeArrow(IArrow a) {
		((Arrow) a).removeArrow();
	}
}
