package org.astakhova.view;

import java.awt.*;
import java.awt.font.*;
import javax.swing.*;
import org.astakhova.data.INode;
import org.astakhova.view.param.SColor;
import java.awt.geom.*;
import org.astakhova.view.param.*;

/**
 * Makes a description of a block
 * @author Astakhova Julia A.
 */
public abstract class Block extends JComponent {
	private INode myNode;
	
	private Rectangle myTextArea;
	private Rectangle2D myStringBounds;

	private final int myOffset = 2;
	private Point myTextPoint = new Point();

	private float myPreferedTextSize;
	private JTextField myEditableComponent;
	
	private Color myColor = SColor.usual();
	
	/**
	 * Creates a new block for the node
	 * @param n a node block is created for
	 */
	public Block(INode n) {
		myNode = n;
		setSize(myNode.getHeight()*100, myNode.getWidth()*100);
	}
	
	/**
	 * Sets a new parameters for area of a node label
	 */
	public void setTextArea() {
		myTextArea = new Rectangle(myNode.getX() + myOffset, 
				myNode.getY() + myOffset,
				myNode.getWidth() - 2*myOffset, 
				myNode.getHeight() - 2*myOffset);
	}
	
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g; 

		g2.setColor(myColor);
		setTextArea();
		myPreferedTextSize = g2.getFont().getSize2D();
		g2.setFont(chooseFont(g2));
		setTextPoint();

		if (myEditableComponent != null) {
			myEditableComponent.setForeground(myColor);
			myEditableComponent.setBounds(myTextArea);
			myEditableComponent.setFont(g2.getFont());
			myEditableComponent.setOpaque(false);
			myEditableComponent.setBorder(null);
		} else
			g.drawString(myNode.getText(), myTextPoint.x, myTextPoint.y);			
		paintBlock(g2);
	}
	
	/**
	 * Paints a form for the the block
	 * @param g graphics to use
	 */
	protected void paintBlock(Graphics g) {
		g.drawRect(myNode.getX(), myNode.getY(), 
				myNode.getWidth(), myNode.getHeight());
	}
	
	private Font chooseFont(Graphics2D g2) {
		Font f = g2.getFont().deriveFont(myPreferedTextSize);
		while (!isTextFontAppropriate(g2) && f.getSize2D() > 1) {
			f = f.deriveFont(f.getSize2D() - 1);
			g2.setFont(f);
		}
		return f;
	}
	
	private boolean isTextFontAppropriate(Graphics2D g2) {
		FontRenderContext frc = new FontRenderContext(null, false, false);
		myStringBounds = g2.getFont().getStringBounds(myNode.getText(), frc);
		return (myTextArea.height >= myStringBounds.getHeight() &&
				myTextArea.width >= myStringBounds.getWidth());
	}
	
	private void setTextPoint() {
		myTextPoint.x = (myTextArea.width - 
				(int)myStringBounds.getWidth())/2 + myTextArea.x;
		myTextPoint.y = (myTextArea.height - 
				(int)myStringBounds.getHeight())/2 + myTextArea.y + 
				(int)myStringBounds.getHeight();
	}
	
	/**
	 * Gets a block parameters
	 * @return a rectangle made up of block parameters
	 */
	public Rectangle getParameters() {
		return new Rectangle(myNode.getX(), myNode.getY(), 
				myNode.getWidth(), myNode.getHeight());
	}
	
	/**
	 * Sets a new parameters for the area of a node label
	 * @param r rectangle made up of parameters
	 */
	public void setTextArea(Rectangle r) {
		myTextArea = r;
	}
	
	public String getText() {
		return myNode.getText();
	}
	
	/**
	 * Selects the block with the color
	 * @param c color to select the block
	 */
	public void setSelected(Color c) {
		myColor = c;
		if (myEditableComponent != null)
			myEditableComponent.setForeground(c);
	}
	
	public INode getNode() {
		return myNode;
	}
	
	/**
	 * Checks if the point is inside the block
	 * @param p point
	 * @return if the point is inside the block
	 */
	public boolean isInside(Point p) {
		Rectangle r = getParameters();
		return (POffset.getX(p.x) >= r.x 
				&& POffset.getX(p.x) <= (r.x + r.width)
				&& POffset.getY(p.y) >= r.y
				&& POffset.getY(p.y) <= (r.y + r.height));
	}
	
	public JTextField getEditableComponent() {
		return myEditableComponent;
	}
	
	public void setEditableComponent(JTextField tf) {
		myEditableComponent = tf;
	}
	
	/**
	 * Checks if the block enters the area
	 *  seted by parameters
	 * @param r rectangle with the parameters
	 * @return
	 */
	public boolean entersArea(Rectangle r) {
		Rectangle b = getParameters();
		return r.contains(b);
	}
}